[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Symbology client library for Python

[![API Version](https://img.shields.io/badge/api-v3.4.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.Symbology)](https://pypi.org/project/fds.sdk.Symbology/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

The FactSet Symbology API provides symbol resolution services, allowing clients to translate market identifiers into various symbology types such as, FactSet Permanent Identifiers, CUSIP, ISIN, SEDOL, Tickers, and LEIs. <p>Factset's Symbology API sits at the center of its hub-and-spoke data model, enabling you to quickly harmonize the expanding catalog of Content APIs.</p><p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 3.4.0
- SDK version: 3.1.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.Symbology==3.1.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.Symbology==3.1.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.Symbology
from fds.sdk.Symbology.api import historical_identifier_resolution_api
from fds.sdk.Symbology.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.Symbology.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.Symbology.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.Symbology.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = historical_identifier_resolution_api.HistoricalIdentifierResolutionApi(api_client)
    identifier_resolution_historical_request = IdentifierResolutionHistoricalRequest(
        data=IdentifierResolutionHistoricalRequestBody(
            ids=GetIdsResolution(["GOOGL-US"]),
            input_symbol_type=GetHistoricalInputSymbolType("BIC"),
            output_symbol_types=GetHistoricalOutputSymbolTypes(["CUSIP","SEDOL"]),
            as_of_date=dateutil_parser('Fri Jan 01 00:00:00 UTC 2010').date(),
        ),
    ) # IdentifierResolutionHistoricalRequest | Request Body for Historical Identifier Resolution.

    try:
        # Returns the specified historical output identifiers for the given input identifier.
        # example passing only required values which don't have defaults set
        api_response = api_instance.batch_historical_identifier_resolution(identifier_resolution_historical_request)

        pprint(api_response)
    except fds.sdk.Symbology.ApiException as e:
        print("Exception when calling HistoricalIdentifierResolutionApi->batch_historical_identifier_resolution: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the specified historical output identifiers for the given input identifier.
    #     api_response, http_status_code, response_headers = api_instance.batch_historical_identifier_resolution_with_http_info(identifier_resolution_historical_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling HistoricalIdentifierResolutionApi->batch_historical_identifier_resolution: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the specified historical output identifiers for the given input identifier.
    #     async_result = api_instance.batch_historical_identifier_resolution_async(identifier_resolution_historical_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling HistoricalIdentifierResolutionApi->batch_historical_identifier_resolution: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the specified historical output identifiers for the given input identifier.
    #     async_result = api_instance.batch_historical_identifier_resolution_with_http_info_async(identifier_resolution_historical_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.Symbology.ApiException as e:
    #     print("Exception when calling HistoricalIdentifierResolutionApi->batch_historical_identifier_resolution: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.Symbology

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.Symbology.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.Symbology

configuration = fds.sdk.Symbology.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*HistoricalIdentifierResolutionApi* | [**batch_historical_identifier_resolution**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/HistoricalIdentifierResolutionApi.md#batch_historical_identifier_resolution) | **POST** /symbology/v3/historical-identifier-resolution | Returns the specified historical output identifiers for the given input identifier.
*HistoricalIdentifierResolutionApi* | [**historical_identifier_resolution**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/HistoricalIdentifierResolutionApi.md#historical_identifier_resolution) | **GET** /symbology/v3/historical-identifier-resolution | Returns the specified historical output identifiers for the given input identifier.
*IdentifierResolutionApi* | [**batch_identifier_resolution**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionApi.md#batch_identifier_resolution) | **POST** /symbology/v3/identifier-resolution | Returns the specified output identifiers for the given input identifier.
*IdentifierResolutionApi* | [**identifier_resolution**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionApi.md#identifier_resolution) | **GET** /symbology/v3/identifier-resolution | Returns the specified output identifiers for the specified input identifier.


## Documentation For Models

 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/ErrorObject.md)
 - [ErrorObjectLinks](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/ErrorObjectLinks.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/ErrorResponse.md)
 - [ErrorResponseHistorical](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/ErrorResponseHistorical.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/ErrorResponseSubErrors.md)
 - [GetHistoricalInputSymbolType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/GetHistoricalInputSymbolType.md)
 - [GetHistoricalOutputSymbolTypes](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/GetHistoricalOutputSymbolTypes.md)
 - [GetIdsResolution](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/GetIdsResolution.md)
 - [GetInputSymbolType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/GetInputSymbolType.md)
 - [GetOutputSymbolTypes](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/GetOutputSymbolTypes.md)
 - [IdentifierResolution](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolution.md)
 - [IdentifierResolutionHistorical](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionHistorical.md)
 - [IdentifierResolutionHistoricalRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionHistoricalRequest.md)
 - [IdentifierResolutionHistoricalRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionHistoricalRequestBody.md)
 - [IdentifierResolutionHistoricalResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionHistoricalResponse.md)
 - [IdentifierResolutionRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionRequest.md)
 - [IdentifierResolutionResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/Symbology/v3/docs/IdentifierResolutionResponse.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.Symbology.apis and fds.sdk.Symbology.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.Symbology.api.default_api import DefaultApi`
- `from fds.sdk.Symbology.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.Symbology
from fds.sdk.Symbology.apis import *
from fds.sdk.Symbology.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

